'-----------------------------------------------------------------------
'  This file is part of the Microsoft Robotics Studio Code Samples.
' 
'  Copyright (C) Microsoft Corporation.  All rights reserved.
'
'  $File: DirectionDialog.vb $ $Revision: 1 $
'-----------------------------------------------------------------------
Imports System
Imports System.ComponentModel
Imports Microsoft.Ccr.Core
Imports Microsoft.Dss.Core
Imports Microsoft.Dss.Core.Attributes
Imports Microsoft.Dss.ServiceModel.Dssp
Imports Microsoft.Dss.ServiceModel.DsspServiceBase
Imports System.Collections.Generic
Imports System.Security.Permissions
Imports W3C.Soap
Imports sm = Microsoft.Dss.Services.SubscriptionManager
Imports Microsoft.Ccr.Adapters.WinForms
Imports soap = W3C.Soap
Imports dssp = Microsoft.Dss.ServiceModel.Dssp

Namespace DirectionDialogVB
    <DisplayName("Direction Dialog (VB)")> _
    <Description("Displays a Windows dialog (compiled with Visual Basic.NET) with 5 buttons on it for direction control.")> _
    <Contract(Contract.Identifier)> _
     Public Class DirectionDialogService
        Inherits DsspServiceBase
        Private _state As DirectionDialogState = New DirectionDialogState()

        <ServicePort("/directiondialog", AllowMultipleInstances:=True)> _
        Private _mainPort As DirectionDialogOperations = New DirectionDialogOperations()

        <Partner("SubMgr", Contract:=sm.Contract.Identifier, CreationPolicy:=PartnerCreationPolicy.CreateAlways, Optional:=False)> _
        Private _submgr As sm.SubscriptionManagerPort = New sm.SubscriptionManagerPort()

        Public Sub New(ByVal creationPort As DsspServiceCreationPort)
            MyBase.New(creationPort)
        End Sub

        Protected Overrides Sub Start()
            _state.Buttons.Add(New Button("Forwards"))
            _state.Buttons.Add(New Button("Backwards"))
            _state.Buttons.Add(New Button("Left"))
            _state.Buttons.Add(New Button("Right"))
            _state.Buttons.Add(New Button("Stop"))

            ' Listen on the main port for requests and call the appropriate handler.
            'ActivateDsspOperationHandlers()
            Activate( _
                Arbiter.Interleave( _
                    New TeardownReceiverGroup( _
                        Arbiter.Receive(Of DsspDefaultDrop)(False, _mainPort, AddressOf DefaultDropHandler) _
                    ) _
                    , New ExclusiveReceiverGroup( _
                        Arbiter.Receive(Of Replace)(True, _mainPort, AddressOf ReplaceHandler) _
                        , Arbiter.Receive(Of ButtonPress)(True, _mainPort, AddressOf ButtonPressHandler) _
                        , Arbiter.Receive(Of ButtonRelease)(True, _mainPort, AddressOf ButtonReleaseHandler) _
                    ) _
                    , New ConcurrentReceiverGroup( _
                        Arbiter.Receive(Of DsspDefaultLookup)(True, _mainPort, AddressOf DefaultLookupHandler) _
                        , Arbiter.Receive(Of GetOperation)(True, _mainPort, AddressOf GetHandler) _
                        , Arbiter.Receive(Of Subscribe)(True, _mainPort, AddressOf SubscribeHandler) _
                    ) _
                ) _
            )

            ' Publish the service to the local service Directory
            DirectoryInsert()

            WinFormsServicePort.Post(New RunForm(AddressOf CreateWinForm))

            ' display HTTP service Uri
            LogInfo(LogGroups.Console, "Service uri: ")

        End Sub
        Function CreateWinForm() As System.Windows.Forms.Form

            Return New Direction(ServiceForwarder(Of DirectionDialogOperations)(ServiceInfo.Service))

        End Function
        Sub GetHandler(ByVal getrequest As GetOperation)
            getrequest.ResponsePort.Post(_state)
        End Sub
        Sub ReplaceHandler(ByVal replace As Replace)
            For Each curr As Button In replace.Body.Buttons
                For Each button As Button In _state.Buttons
                    If button.Name = curr.Name Then
                        button.Pressed = curr.Pressed
                    End If
                Next
            Next
            replace.ResponsePort.Post(DefaultReplaceResponseType.Instance)
            SendNotification(Of Replace)(_submgr, _state)
        End Sub
        Sub ButtonPressHandler(ByVal buttonPressed As ButtonPress)
            Dim button As Button = Nothing
            For Each test As Button In _state.Buttons
                If test.Name = buttonPressed.Body.Name Then
                    button = test
                End If
            Next
            If Object.ReferenceEquals(button, Nothing) Then
                buttonPressed.ResponsePort.Post(Fault.FromCodeSubcodeReason( _
                    soap.FaultCodes.Receiver, _
                    DsspFaultCodes.UnknownEntry, _
                    "No such button: " + buttonPressed.Body.Name _
                    ) _
                )
            Else
                button.Pressed = True
                buttonPressed.ResponsePort.Post(DefaultUpdateResponseType.Instance)
                SendNotification(_submgr, buttonPressed)
            End If
        End Sub
        Sub ButtonReleaseHandler(ByVal buttonReleased As ButtonRelease)
            Dim button As Button = Nothing
            For Each test As Button In _state.Buttons
                If test.Name = buttonReleased.Body.Name Then
                    button = test
                End If
            Next
            If Object.ReferenceEquals(button, Nothing) Then
                buttonReleased.ResponsePort.Post(Fault.FromCodeSubcodeReason( _
                    soap.FaultCodes.Receiver, _
                    DsspFaultCodes.UnknownEntry, _
                    "No such button: " + buttonReleased.Body.Name _
                    ) _
                )
            Else
                button.Pressed = False
                buttonReleased.ResponsePort.Post(DefaultUpdateResponseType.Instance)
                SendNotification(_submgr, buttonReleased)
            End If
        End Sub
        Sub SubscribeHandler(ByVal subscribe As Subscribe)
            SubscribeHelper(_submgr, subscribe.Body, subscribe.ResponsePort)
        End Sub

    End Class
End Namespace


