//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: FollowerTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------

using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using System;
using System.Collections.Generic;
using W3C.Soap;

using bumper = Microsoft.Robotics.Services.ContactSensor.Proxy;
using drive = Microsoft.Robotics.Services.Drive.Proxy;
using sicklrf = Microsoft.Robotics.Services.Sensors.SickLRF.Proxy;

using follower = Microsoft.Robotics.Services.Sample.Follower;
using System.ComponentModel;


namespace Microsoft.Robotics.Services.Sample.Follower
{
    
    /// <summary>
    /// Follower Contract class
    /// </summary>
    public sealed class Contract
    {
        /// <summary>
        /// The Dss Service contract
        /// </summary>
        public const String Identifier = "http://schemas.microsoft.com/robotics/2007/04/follower.html";
    }

    /// <summary>
    /// Follower Main Operations Port
    /// </summary>
    class FollowerOperations : PortSet<
        DsspDefaultLookup,
        DsspDefaultDrop,
        SetMotorActivate,
        SetRobotBehavior,
        Get
        >
    {
    }

    /// <summary>
    /// Get Operation
    /// </summary>
    [Description("Gets the current state of this service.")]
    public class Get : Get<GetRequestType, PortSet<FollowerState, Fault>>
    {
    }

    /// <summary>
    /// Motor Activation
    /// </summary>
    [Description("Activates or deactivates the drive motors.")]
    public class SetMotorActivate : Replace<BoolFlag, PortSet<DefaultReplaceResponseType, Fault>>
    {
        /// <summary>
        /// Motor Activation
        /// </summary>
        public SetMotorActivate()
        { }

        /// <summary>
        /// Motor Activation
        /// </summary>
        /// <param name="activate"></param>
        public SetMotorActivate(bool activate)
        {
            this.Body = new BoolFlag(activate);
        }
    }

    /// <summary>
    /// Request a Robot Behavior
    /// </summary>
    [Description("Sets a specific behavior.")]
    public class SetRobotBehavior : Update<RequestRobotBehavior, PortSet<DefaultUpdateResponseType, Fault>>
    {
        /// <summary>
        /// Request a Robot Behavior
        /// </summary>
        public SetRobotBehavior()
        { }

        /// <summary>
        /// Request a Robot Behavior
        /// </summary>
        /// <param name="behavior"></param>
        public SetRobotBehavior(RobotBehavior behavior)
        {
            this.Body = new RequestRobotBehavior(behavior);
        }
    }

    /// <summary>
    /// DisableSpeechUpdate
    /// </summary>
    /// <remarks>Internal message</remarks>
    public class DisableSpeechUpdate : Update<BoolFlag, PortSet<DefaultUpdateResponseType, Fault>>
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public DisableSpeechUpdate()
        { }
        /// <summary>
        /// Constructor with a boolean flag
        /// </summary>
        public DisableSpeechUpdate(bool flag)
        {
            this.Body = new BoolFlag(flag);
        }
    }

    /// <summary>
    /// Contains a boolean flag
    /// </summary>
    [DataContract]
    public class BoolFlag
    {
        /// <summary>
        /// A Boolean Flag
        /// </summary>
        [DataMember, DataMemberConstructor]
        [Description("Specifies an On or Off flag.")]
        public bool OnOff;

        /// <summary>
        /// Constructor
        /// </summary>
        public BoolFlag()
        { }
        /// <summary>
        /// Constructor with a boolean flag
        /// </summary>
        public BoolFlag(bool flag)
        {
            OnOff = flag;
        }
    }

}
