using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using Microsoft.Dss.ServiceModel.Dssp;

using sicklrf = Microsoft.Robotics.Services.Sensors.SickLRF.Proxy;
using drive = Microsoft.Robotics.Services.Drive.Proxy;

namespace Microsoft.Robotics.Services.Sample.Follower
{
    partial class MyForm : Form
    {
        FollowerOperations _mainPort;
        drive.DriveOperations _drivePort;
        bool keyDown = false;

        public MyForm(FollowerOperations mainPort, drive.DriveOperations drivePort)
        {
            InitializeComponent();

            _mainPort = mainPort;
            _drivePort = drivePort;
        }

        public void UpdateJoystickAxes(Axes axes)
        {
            int x = axes.X;
            int y = -axes.Y;

            lblX.Text = x.ToString();
            lblY.Text = y.ToString();

            DrawJoystick(x, y);
            int left;
            int right;

            if (y > -100)
            {
                left = y + axes.X / 4;
                right = y - axes.X / 4;
            }
            else
            {
                left = y - axes.X / 4;
                right = y + axes.X / 4;
            }
            
            left = left * 750 / 1250;
            right = right * 750 / 1250;

            double MOTOR_POWER_SCALE_FACTOR = 0.0005;
            _drivePort.SetDrivePower(new drive.SetDrivePowerRequest((double)left * MOTOR_POWER_SCALE_FACTOR,(double)right * MOTOR_POWER_SCALE_FACTOR));

        }

        private void picJoystick_MouseLeave(object sender, EventArgs e)
        {
            UpdateJoystickAxes(new Axes());
        }

        private void picJoystick_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                int x, y;
                x = Math.Min(picJoystick.Width, Math.Max(e.X, 0));
                y = Math.Min(picJoystick.Height, Math.Max(e.Y, 0));

                x = x * 2000 / picJoystick.Width - 1000;
                y = y * 2000 / picJoystick.Height - 1000;

                Axes axes = new Axes();
                axes.X = x;
                axes.Y = y;

                UpdateJoystickAxes(axes);
            }
        }

        private void picJoystick_MouseUp(object sender, MouseEventArgs e)
        {
            picJoystick_MouseLeave(sender, e);
        }


        private void DrawJoystick(int x, int y)
        {
            Bitmap bmp = new Bitmap(picJoystick.Width, picJoystick.Height);
            Graphics g = Graphics.FromImage(bmp);

            int width = bmp.Width - 1;
            int height = bmp.Height - 1;

            g.Clear(Color.Transparent);
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            GraphicsPath path = new GraphicsPath();
            path.AddEllipse(0, 0, width, height);

            PathGradientBrush pathBrush = new PathGradientBrush(path);
            pathBrush.CenterPoint = new PointF(width / 3f, height / 3f);
            pathBrush.CenterColor = Color.White;
            pathBrush.SurroundColors = new Color[] { Color.LightGray };

            g.FillPath(pathBrush, path);
            g.DrawPath(Pens.Black, path);

            int partial = y * height / 2200;
            if (partial > 0)
            {
                g.DrawArc(Pens.Black,
                    0,
                    height / 2 - partial,
                    width,
                    2 * partial,
                    180,
                    180);
            }
            else if (partial == 0)
            {
                g.DrawLine(Pens.Black, 0, height / 2, width, height / 2);
            }
            else
            {
                g.DrawArc(Pens.Black,
                    0,
                    height / 2 + partial,
                    width,
                    -2 * partial,
                    0,
                    180);
            }

            partial = x * width / 2200;
            if (partial > 0)
            {
                g.DrawArc(Pens.Black,
                    width / 2 - partial,
                    0,
                    2 * partial,
                    height,
                    270,
                    180);
            }
            else if (partial == 0)
            {
                g.DrawLine(Pens.Black, width / 2, 0, width / 2, height);
            }
            else
            {
                g.DrawArc(Pens.Black,
                    width / 2 + partial,
                    0,
                    -2 * partial,
                    height,
                    90,
                    180);
            }

            picJoystick.Image = bmp;
        }

        private void MyForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (keyDown == false)
            {
                keyDown = true;

                _drivePort.EnableDrive(new drive.EnableDriveRequest(true));
                switch (e.KeyCode)
                {
                    case Keys.NumPad8:
                    case Keys.I:
                        _drivePort.SetDrivePower(new drive.SetDrivePowerRequest(0.1, 0.1));
                        break;

                    case Keys.NumPad4:
                    case Keys.J:
                        _drivePort.SetDrivePower(new drive.SetDrivePowerRequest(-0.05, 0.05));
                        break;

                    case Keys.NumPad6:
                    case Keys.L:
                        _drivePort.SetDrivePower(new drive.SetDrivePowerRequest(0.05, -0.05));
                        break;

                    case Keys.NumPad2:
                    case Keys.M:
                        _drivePort.SetDrivePower(new drive.SetDrivePowerRequest(-0.1, -0.1));
                        break;

                    case Keys.NumPad5:
                    case Keys.K:
                        _drivePort.SetDrivePower(new drive.SetDrivePowerRequest(0, 0));
                        break;
                }
            }
        }

        private void MyForm_KeyPress(object sender, KeyPressEventArgs e)
        {
        }

        private void MyForm_KeyUp(object sender, KeyEventArgs e)
        {
            if (keyDown)
            {
                keyDown = false;
                _drivePort.SetDrivePower(new drive.SetDrivePowerRequest(0, 0));
            }
        }
        
        bool disableMotor = true;
        private void button3_Click(object sender, EventArgs e)
        {
            disableMotor ^= true;
           
            if (disableMotor == false)
                robotStatus.Text = " Activated";
            else
                robotStatus.Text = "Deactivated";

            _drivePort.SetDrivePower(new drive.SetDrivePowerRequest(0, 0));
            _mainPort.Post(new SetMotorActivate(disableMotor));

        }

        private void button1_Click(object sender, EventArgs e)
        {
            //Avoid
            _mainPort.Post(new SetRobotBehavior(RobotBehavior.Avoid)); 
        }
  
        private void button2_Click(object sender, EventArgs e)
        {
            //GoThere
            _mainPort.Post(new SetRobotBehavior(RobotBehavior.GoThere)); 
        }

        private void button4_Click(object sender, EventArgs e)
        {
            //Stop
            _mainPort.Post(new SetRobotBehavior(RobotBehavior.Stop)); 
        }

        private void button5_Click(object sender, EventArgs e)
        {
            //FollowMe
            _mainPort.Post(new SetRobotBehavior(RobotBehavior.FollowMe)); 
        }

        protected override void OnClosed(EventArgs e)
        {
            base.OnClosed(e);

            _mainPort.Post(new DsspDefaultDrop());
        }

    }

    class Axes
    {
        private int _x;
        public int X
        {
            get { return _x; }
            set { _x = value; }
        }

        private int _y;
        public int Y
        {
            get { return _y; }
            set { _y = value; }
        }
    }
}