//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: AtomUtil.cs $ $Revision: 1 $
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;

namespace Microsoft.Robotics.Services.Syndication.Atom
{
    internal class AtomUtilities
    {
        /// <summary>
        /// Helper for adding a query token to array of tokens
        /// </summary>
        /// <param name="arg">New query token</param>
        /// <param name="result">List of tokens</param>
        static void AddNewToken(ref StringBuilder arg, ref List<string> result)
        {
            if (arg.Length > 0)
            {
                result.Add(arg.ToString());
            }
            arg.Length = 0;
        }

        /// <summary>
        /// Splits the query string into individual tokens. The tokenizer
        /// accepts use of quotes (") to allow for whitespace in tokens.
        /// </summary>
        /// <param name="query">Query string to tokenize</param>
        /// <returns>Array of query tokens</returns>
        static public string[] QueryTokenizer(string query)
        {
            List<string> result = new List<string>();
            bool inQuote = false;
            StringBuilder arg = new StringBuilder();
            foreach (char c in query)
            {
                if (c == '"')
                {
                    if (inQuote)
                    {
                        AddNewToken(ref arg, ref result);
                    }
                    inQuote = !inQuote;
                }
                else if (Char.IsControl(c) || Char.IsPunctuation(c))
                {
                    if (inQuote)
                    {
                        arg.Append(' ');
                    }
                    else
                    {
                        AddNewToken(ref arg, ref result);
                    }
                }
                else if (Char.IsWhiteSpace(c))
                {
                    if (inQuote)
                    {
                        arg.Append(c);
                    }
                    else
                    {
                        AddNewToken(ref arg, ref result);
                    }
                }
                else
                {
                    arg.Append(c);
                }
            }
            AddNewToken(ref arg, ref result);
            return result.ToArray();
        }

    }
}
