//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: Dialog.cs $ $Revision: 1 $
//-----------------------------------------------------------------------
#if !URT_MINCLR
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Security.Permissions;
using xml = System.Xml;
using sm = Microsoft.Dss.Services.SubscriptionManager;
using System.Text;
using Microsoft.Ccr.Adapters.WinForms;
using W3C.Soap;

namespace Microsoft.Robotics.Services.Sample.Dialog
{

    [DisplayName("Simple Dialog")]
    [Description("Provides access to a simple dialog box service.")]
    [Contract(Contract.Identifier)]
    public class DialogService : DsspServiceBase
    {
        [ServicePort("/dialog", AllowMultipleInstances = true)]
        private DialogOperations _mainPort = new DialogOperations();

        /// <summary>
        /// Default Service Constructor
        /// </summary>
        public DialogService(DsspServiceCreationPort creationPort)
            :
                base(creationPort)
        {
            
        }

        /// <summary>
        /// Service Start
        /// </summary>
        protected override void Start()
        {
            // Listen on the main port for requests and call the appropriate handler.
            ActivateDsspOperationHandlers();

            // Publish the service to the local service Directory
            DirectoryInsert();
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> AlertHandler(Alert alert)
        {
            SuccessFailurePort result = new SuccessFailurePort();

            RunForm runForm = new RunForm(
                delegate()
                {
                    AlertForm form = new AlertForm(result);
                    form.Message = alert.Body.Message;

                    return form;
                }
            );
            WinFormsServicePort.Post(runForm);

            yield return Arbiter.Choice(
                runForm.pResult,
                delegate(SuccessResult success){},
                delegate(Exception e)
                {
                    result.Post(e);
                }
            );

            yield return Arbiter.Choice(
                result,
                delegate(SuccessResult success)
                {
                    alert.ResponsePort.Post(DefaultSubmitResponseType.Instance);
                },
                delegate(Exception e)
                {
                    Fault fault = Fault.FromException(e);
                    LogError(null, "Error in Alert Handler", fault);
                    alert.ResponsePort.Post(fault);
                }
            );
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> ConfirmHandler(Confirm confirm)
        {
            PortSet<bool, Exception> result = new PortSet<bool, Exception>();

            RunForm runForm = new RunForm(
                delegate()
                {
                    ConfirmForm form = new ConfirmForm(result);
                    form.Message = confirm.Body.Message;

                    return form;
                }
            );
            WinFormsServicePort.Post(runForm);

            yield return Arbiter.Choice(
                runForm.pResult,
                delegate(SuccessResult success) { },
                delegate(Exception e)
                {
                    result.Post(e);
                }
            );

            yield return Arbiter.Choice(
                result,
                delegate(bool confirmed)
                {
                    ConfirmResponse response = new ConfirmResponse();
                    response.Confirmed = confirmed;

                    confirm.ResponsePort.Post(response);
                },
                delegate(Exception e)
                {
                    Fault fault = Fault.FromException(e);
                    LogError(null, "Error in Confirm Handler", fault);
                    confirm.ResponsePort.Post(fault);
                }
            );
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> PromptHandler(Prompt prompt)
        {
            PortSet<string, Exception> result = new PortSet<string, Exception>();

            RunForm runForm = new RunForm(
                delegate()
                {
                    PromptForm form = new PromptForm(result);
                    form.Message = prompt.Body.Message;
                    form.DefaultValue = prompt.Body.DefaultValue;

                    return form;
                }
            );
            WinFormsServicePort.Post(runForm);

            yield return Arbiter.Choice(
                runForm.pResult,
                delegate(SuccessResult success) { },
                delegate(Exception e)
                {
                    result.Post(e);
                }
            );

            yield return Arbiter.Choice(
                result,
                delegate(string value)
                {
                    PromptResponse response = new PromptResponse();
                    response.TextData = value;

                    prompt.ResponsePort.Post(response);
                },
                delegate(Exception e)
                {
                    Fault fault = Fault.FromException(e);
                    LogError(null, "Error in Prompt Handler", fault);
                    prompt.ResponsePort.Post(fault);
                }
            );
        }
    }
}
#endif
