//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: Sound.cs $ $Revision: 1 $
//-----------------------------------------------------------------------
#if !URT_MINCLR
using System;
using System.Collections.Generic;
using System.Text;
using System.Media;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using Microsoft.Dss.Core.Attributes;
using System.ComponentModel;
using System.Security;
using System.Security.Permissions;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Ccr.Core;
using W3C.Soap;

namespace Microsoft.Robotics.Services.Sample.Sound
{
    [DisplayName("Sound Player")]
    [Description("Plays .wav files and system sounds.")]
    [Contract(Contract.Identifier)]
    public class SoundService : DsspServiceBase
    {
        [ServicePort("sound", AllowMultipleInstances = false)]
        SoundOperations _mainPort = new SoundOperations();

        public SoundService(DsspServiceCreationPort create)
            : base(create)
        {
            
        }

        protected override void Start()
        {
            ActivateDsspOperationHandlers();
            DirectoryInsert();
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> PlayHandler(Play play)
        {
            SoundPlayer player = new SoundPlayer(play.Body.Filename);

            if (play.Body.Synchronous)
            {
                player.Load();
                player.PlaySync();
            }
            else
            {
                player.Play();
            }
            play.ResponsePort.Post(DefaultSubmitResponseType.Instance);
            yield break;
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> AsteriskHandler(Asterisk asterisk)
        {
            return PlaySound(asterisk.Body.Synchronous, asterisk.ResponsePort, SystemSounds.Asterisk);
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> BeepHandler(Beep beep)
        {
            return PlaySound(beep.Body.Synchronous, beep.ResponsePort, SystemSounds.Beep);
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> ExclamationHandler(Exclamation exclamation)
        {
            return PlaySound(exclamation.Body.Synchronous, exclamation.ResponsePort, SystemSounds.Exclamation);
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> HandHandler(Hand hand)
        {
            return PlaySound(hand.Body.Synchronous, hand.ResponsePort, SystemSounds.Hand);
        }

        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> QuestionHandler(Question question)
        {
            return PlaySound(question.Body.Synchronous, question.ResponsePort, SystemSounds.Question);
        }

        private IEnumerator<ITask> PlaySound(bool synchronous, PortSet<DefaultSubmitResponseType, Fault> responsePort, SystemSound systemSound)
        {
            if (synchronous)
            {
                systemSound.Play();
            }
            else
            {
                Spawn(delegate()
                    {
                        systemSound.Play();
                    }
                );
            }
            responsePort.Post(DefaultSubmitResponseType.Instance);
            yield break;
        }
    }
}
#endif